package org.jboss.soa.esb.samples.quickstart.bh;

import java.rmi.RemoteException;
import java.util.List;

import org.apache.commons.configuration.ConfigurationException;

import org.apache.juddi.Registry;
import org.apache.juddi.api.impl.JUDDIApiImpl;
import org.apache.juddi.api.impl.UDDISecurityImpl;
import org.apache.juddi.api_v3.DeletePublisher;
import org.apache.juddi.api_v3.GetPublisherDetail;
import org.apache.juddi.api_v3.Publisher;
import org.apache.juddi.api_v3.PublisherDetail;
import org.apache.juddi.api_v3.SavePublisher;
import org.apache.juddi.jaxb.EntityCreator;
import org.apache.juddi.v3.error.InvalidKeyPassedException;

import org.apache.log4j.Logger;

import org.testng.Assert;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import org.uddi.api_v3.AuthToken;
import org.uddi.api_v3.DiscardAuthToken;
import org.uddi.api_v3.GetAuthToken;
import org.uddi.v3_service.DispositionReportFaultMessage;
import org.uddi.v3_service.UDDISecurityPortType;

@Test(enabled = true)
public class DatabaseBlackholeIntroducer {

	private static Logger logger = Logger.getLogger(DatabaseBlackholeIntroducer.class);

	private static String authInfo = null;
	
	private JUDDIApiImpl publisher = new JUDDIApiImpl();
	private UDDISecurityPortType security = new UDDISecurityImpl();

	@BeforeClass
	public static void startRegistry() throws ConfigurationException {
		Registry.start();
	}

	@AfterClass
	public static void stopRegistry() throws ConfigurationException {
		Registry.stop();
	}

	@Test
	public void testJoePublisher() {
		savePublisher("joepublisher", "juddi_data/joe_publisher.xml");
		deletePublisher("joepublisher");
	}

	private void savePublisher(String publisherId, String publisherXML) {
		try {
			String rootPublisherStr = "root";
			authInfo = getAuthToken(rootPublisherStr, "");
			logger.debug("Saving new publisher: " + publisherXML);
			SavePublisher sp = new SavePublisher();
			sp.setAuthInfo(authInfo);
			Publisher pubIn = (Publisher) EntityCreator.buildFromDoc(publisherXML, "org.apache.juddi.api_v3");
			sp.getPublisher().add(pubIn);
			publisher.savePublisher(sp);

			// Now get the entity and check the values
			GetPublisherDetail gp = new GetPublisherDetail();
			gp.getPublisherId().add(publisherId);
			gp.setAuthInfo(authInfo);
			PublisherDetail pd = publisher.getPublisherDetail(gp);
			List<Publisher> pubOutList = pd.getPublisher();
			Publisher pubOut = pubOutList.get(0);

			Assert.assertEquals(pubIn.getAuthorizedName(), pubOut.getAuthorizedName());
			Assert.assertEquals(pubIn.getPublisherName(), pubOut.getPublisherName());
			Assert.assertEquals(pubIn.getEmailAddress(), pubOut.getEmailAddress());
			Assert.assertEquals(pubIn.getIsAdmin(), pubOut.getIsAdmin());
			Assert.assertEquals(pubIn.getIsEnabled(), pubOut.getIsEnabled());
			Assert.assertEquals(pubIn.getMaxBindingsPerService(), pubOut.getMaxBindingsPerService());
			Assert.assertEquals(pubIn.getMaxBusinesses(), pubOut.getMaxBusinesses());
			Assert.assertEquals(pubIn.getMaxServicePerBusiness(), pubOut.getMaxServicePerBusiness());
			Assert.assertEquals(pubIn.getMaxTModels(), pubOut.getMaxTModels());

			logger.debug("Querying for publisher: " + publisherXML);
			// Querying for this publisher to make sure it's really gone
			// We're expecting a invalid Key exception at this point.
			PublisherDetail pdBeforeDelete = null;
			try {
				pdBeforeDelete = publisher.getPublisherDetail(gp);
				Assert.assertNotNull(pdBeforeDelete);
			} catch (InvalidKeyPassedException e) {
				Assert.fail("We expected to find publisher " + publisherXML);
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Assert.fail("No exception should be thrown");
		} finally {
			try {
				discardAuthToken(authInfo);
			} catch (Exception e) {
				// be quiet !!!
			}
		}
	}

	private void deletePublisher(String publisherId) {
		try {
			String rootPublisherStr = "root";
			authInfo = getAuthToken(rootPublisherStr, "");
			logger.debug("Delete publisher: " + publisherId);
			// Now deleting this publisher
			DeletePublisher dp = new DeletePublisher();
			dp.setAuthInfo(authInfo);
			dp.getPublisherId().add(publisherId);
			publisher.deletePublisher(dp);

			logger.info("Querying for publisher: " + publisherId + " after deletion.");
			// Querying for this publisher to make sure it's really gone
			// We're expecting a invalid Key exception at this point.
			GetPublisherDetail gp = new GetPublisherDetail();
			gp.getPublisherId().add(publisherId);
			gp.setAuthInfo(authInfo);
			PublisherDetail pdAfterDelete = null;
			try {
				pdAfterDelete = publisher.getPublisherDetail(gp);
				Assert.fail("We did not expect to find this publisher anymore.");
			} catch (InvalidKeyPassedException e) {
				Assert.assertNull(pdAfterDelete);
			}

		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Assert.fail("No exception should be thrown");
		} finally {
			try {
				discardAuthToken(authInfo);
			} catch (Exception e) {
				// be quiet !!!
			}
		}
	}

	private String getAuthToken(String pubId, String cred) throws RemoteException, DispositionReportFaultMessage {
		GetAuthToken ga = new GetAuthToken();
		ga.setUserID(pubId);
		ga.setCred(cred);
		AuthToken token = security.getAuthToken(ga);
		return token.getAuthInfo();
	}
	
	private void discardAuthToken(String authInfo) throws RemoteException, DispositionReportFaultMessage {
		DiscardAuthToken dat = new DiscardAuthToken();
		dat.setAuthInfo(authInfo);
		security.discardAuthToken(dat);
	}
	
}
